@@ -1680,11 +1680,19 @@ static void computeSecondInPair(ReadSet * reads)
 					phase = 0;
 				}
 			}
-			else
+			else {
 				phase = 1;
+				if (index > 0 && previousCat & 1 && !isSecondInPair(reads, index - 1))
+				    reads->categories[index - 1] = (reads->categories[index - 1] / 2) * 2;
+			}
 		}
 		previousCat = currentCat;
 	}
+
+	// Safeguard against odd sets of reads
+	if (!isSecondInPair(reads, reads->readCount - 1)) {
+		reads->categories[reads->readCount - 1] = (reads->categories[reads->readCount - 1] / 2) * 2;
+	}
 }
 
 void detachDubiousReads(ReadSet * reads, boolean * dubiousReads)
@@ -1913,7 +1921,7 @@ void destroyReadSet(ReadSet * reads)
 ShortLength *getSequenceLengths(ReadSet * reads, int wordLength)
 {
 	ShortLength *lengths = callocOrExit(reads->readCount, ShortLength);
-	ShortLength index;
+	IDnum index;
 	int lengthOffset = wordLength - 1;
 
 	for (index = 0; index < reads->readCount; index++)
